import json


class Model:
    name = None
    edit_format = None
    max_context_tokens = 0
    tokenizer = None
    max_chat_history_tokens = 1024

    always_available = False
    use_repo_map = False
    send_undo_reply = False

    prompt_price = None
    completion_price = None

    @classmethod
    def create(cls, name, client=None):
        from .openai import OpenAIModel
        from .openrouter import OpenRouterModel

        if client and client.base_url.host == "openrouter.ai":
            return OpenRouterModel(client, name)
        return OpenAIModel(name)

    def __str__(self):
        return self.name

    @staticmethod
    def strong_model():
        return Model.create("gpt-4")

    @staticmethod
    def weak_model():
        return Model.create("gpt-3.5-turbo-1106")

    @staticmethod
    def commit_message_models():
        return [Model.weak_model()]

    def token_count(self, messages):
        if not self.tokenizer:
            return

        if type(messages) is str:
            msgs = messages
        else:
            msgs = json.dumps(messages)

        return len(self.tokenizer.encode(msgs))



import unittest
from unittest.mock import MagicMock

from aider.models import Model, OpenRouterModel


class TestModels(unittest.TestCase):
    def test_max_context_tokens(self):
        model = Model.create("gpt-3.5-turbo")
        self.assertEqual(model.max_context_tokens, 4 * 1024)

        model = Model.create("gpt-3.5-turbo-16k")
        self.assertEqual(model.max_context_tokens, 16 * 1024)

        model = Model.create("gpt-3.5-turbo-1106")
        self.assertEqual(model.max_context_tokens, 16 * 1024)

        model = Model.create("gpt-4")
        self.assertEqual(model.max_context_tokens, 8 * 1024)

        model = Model.create("gpt-4-32k")
        self.assertEqual(model.max_context_tokens, 32 * 1024)

        model = Model.create("gpt-4-0101")
        self.assertEqual(model.max_context_tokens, 8 * 1024)

        model = Model.create("gpt-4-32k-2123")
        self.assertEqual(model.max_context_tokens, 32 * 1024)

    def test_openrouter_model_properties(self):
        client = MagicMock()
        client.models.list.return_value = {
            "data": [
                {
                    "id": "openai/gpt-4",
                    "object": "model",
                    "context_length": "8192",
                    "pricing": {"prompt": "0.00006", "completion": "0.00012"},
                }
            ]
        }
        client.models.list.return_value = type(
            "", (), {"data": client.models.list.return_value["data"]}
        )()

        model = OpenRouterModel(client, "gpt-4")
        self.assertEqual(model.name, "openai/gpt-4")
        self.assertEqual(model.max_context_tokens, 8192)
        self.assertEqual(model.prompt_price, 0.06)
        self.assertEqual(model.completion_price, 0.12)


if __name__ == "__main__":
    unittest.main()
